@@ -9,17 +9,16 @@
 #include "iface_internal.h"
 
 /* Create a low level structure from record */
-static int iface_from_record (
-	sepol_handle_t* handle,
-	const policydb_t* policydb,
-	ocontext_t** iface,
-	const sepol_iface_t* record) {
+static int iface_from_record(sepol_handle_t * handle,
+			     const policydb_t * policydb,
+			     ocontext_t ** iface, const sepol_iface_t * record)
+{
 
-	ocontext_t* tmp_iface = NULL;
-	context_struct_t* tmp_con = NULL;
+	ocontext_t *tmp_iface = NULL;
+	context_struct_t *tmp_con = NULL;
 
 	tmp_iface = (ocontext_t *) calloc(1, sizeof(ocontext_t));
-	if (!tmp_iface) 	
+	if (!tmp_iface)
 		goto omem;
 
 	/* Name */
@@ -28,8 +27,8 @@ static int iface_from_record (
 		goto omem;
 
 	/* Interface Context */
-	if (context_from_record(handle, policydb, 
-		&tmp_con, sepol_iface_get_ifcon(record)) < 0)
+	if (context_from_record(handle, policydb,
+				&tmp_con, sepol_iface_get_ifcon(record)) < 0)
 		goto err;
 	context_cpy(&tmp_iface->context[0], tmp_con);
 	context_destroy(tmp_con);
@@ -37,8 +36,8 @@ static int iface_from_record (
 	tmp_con = NULL;
 
 	/* Message Context */
-	if (context_from_record(handle, policydb, 
-		&tmp_con, sepol_iface_get_msgcon(record)) < 0)
+	if (context_from_record(handle, policydb,
+				&tmp_con, sepol_iface_get_msgcon(record)) < 0)
 		goto err;
 	context_cpy(&tmp_iface->context[1], tmp_con);
 	context_destroy(tmp_con);
@@ -48,10 +47,10 @@ static int iface_from_record (
 	*iface = tmp_iface;
 	return STATUS_SUCCESS;
 
-	omem:
+      omem:
 	ERR(handle, "out of memory");
 
-	err:
+      err:
 	if (tmp_iface != NULL) {
 		free(tmp_iface->u.name);
 		context_destroy(&tmp_iface->context[0]);
@@ -64,18 +63,17 @@ static int iface_from_record (
 	return STATUS_ERR;
 }
 
-static int iface_to_record (
-	sepol_handle_t* handle,
-	const policydb_t* policydb,
-	ocontext_t* iface,
-        sepol_iface_t** record) {
+static int iface_to_record(sepol_handle_t * handle,
+			   const policydb_t * policydb,
+			   ocontext_t * iface, sepol_iface_t ** record)
+{
 
-	char* name = iface->u.name;
-	context_struct_t* ifcon = &iface->context[0];
-	context_struct_t* msgcon = &iface->context[1];
+	char *name = iface->u.name;
+	context_struct_t *ifcon = &iface->context[0];
+	context_struct_t *msgcon = &iface->context[1];
 
-	sepol_context_t* tmp_con = NULL;
-	sepol_iface_t* tmp_record = NULL;
+	sepol_context_t *tmp_con = NULL;
+	sepol_iface_t *tmp_record = NULL;
 
 	if (sepol_iface_create(handle, &tmp_record) < 0)
 		goto err;
@@ -89,7 +87,7 @@ static int iface_to_record (
 		goto err;
 	sepol_context_free(tmp_con);
 	tmp_con = NULL;
-	
+
 	if (context_to_record(handle, policydb, msgcon, &tmp_con) < 0)
 		goto err;
 	if (sepol_iface_set_msgcon(handle, tmp_record, tmp_con) < 0)
@@ -100,7 +98,7 @@ static int iface_to_record (
 	*record = tmp_record;
 	return STATUS_SUCCESS;
 
-	err:
+      err:
 	ERR(handle, "could not convert interface %s to record", name);
 	sepol_context_free(tmp_con);
 	sepol_iface_free(tmp_record);
@@ -108,16 +106,15 @@ static int iface_to_record (
 }
 
 /* Check if an interface exists */
-int sepol_iface_exists (
-	sepol_handle_t* handle,
-	const sepol_policydb_t* p,
-	const sepol_iface_key_t* key,
-	int* response) {
+int sepol_iface_exists(sepol_handle_t * handle,
+		       const sepol_policydb_t * p,
+		       const sepol_iface_key_t * key, int *response)
+{
 
 	const policydb_t *policydb = &p->p;
 	ocontext_t *c, *head;
 
-	const char* name;
+	const char *name;
 	sepol_iface_key_unpack(key, &name);
 
 	head = policydb->ocontexts[OCON_NETIF];
@@ -134,16 +131,15 @@ int sepol_iface_exists (
 }
 
 /* Query an interface */
-int sepol_iface_query (
-	sepol_handle_t* handle,
-	const sepol_policydb_t* p,
-	const sepol_iface_key_t* key,
-	sepol_iface_t** response) {
+int sepol_iface_query(sepol_handle_t * handle,
+		      const sepol_policydb_t * p,
+		      const sepol_iface_key_t * key, sepol_iface_t ** response)
+{
 
 	const policydb_t *policydb = &p->p;
 	ocontext_t *c, *head;
 
-	const char* name;
+	const char *name;
 	sepol_iface_key_unpack(key, &name);
 
 	head = policydb->ocontexts[OCON_NETIF];
@@ -155,27 +151,27 @@ int sepol_iface_query (
 
 			return STATUS_SUCCESS;
 		}
-	} 
+	}
 
 	*response = NULL;
 	return STATUS_SUCCESS;
 
-	err: 
+      err:
 	ERR(handle, "could not query interface %s", name);
 	return STATUS_ERR;
 }
 
 /* Load an interface into policy */
-int sepol_iface_modify(
-	sepol_handle_t* handle,
-	sepol_policydb_t* p,
-	const sepol_iface_key_t* key, 
-	const sepol_iface_t* data) {
+int sepol_iface_modify(sepol_handle_t * handle,
+		       sepol_policydb_t * p,
+		       const sepol_iface_key_t * key,
+		       const sepol_iface_t * data)
+{
 
 	policydb_t *policydb = &p->p;
 	ocontext_t *head, *prev, *c, *iface = NULL;
 
-	const char* name;
+	const char *name;
 	sepol_iface_key_unpack(key, &name);
 
 	if (iface_from_record(handle, policydb, &iface, data) < 0)
@@ -200,14 +196,14 @@ int sepol_iface_modify(
 			return STATUS_SUCCESS;
 		}
 		prev = c;
-	}	
-		
+	}
+
 	/* Attach to context list */
 	iface->next = policydb->ocontexts[OCON_NETIF];
 	policydb->ocontexts[OCON_NETIF] = iface;
 	return STATUS_SUCCESS;
 
-	err:
+      err:
 	ERR(handle, "error while loading interface %s", name);
 
 	if (iface != NULL) {
@@ -220,17 +216,16 @@ int sepol_iface_modify(
 }
 
 /* Return the number of interfaces */
-extern int sepol_iface_count(
-	sepol_handle_t* handle,
-	const sepol_policydb_t* p,
-	unsigned int* response) {
+extern int sepol_iface_count(sepol_handle_t * handle,
+			     const sepol_policydb_t * p, unsigned int *response)
+{
 
 	unsigned int count = 0;
 	ocontext_t *c, *head;
-	const policydb_t* policydb = &p->p;
+	const policydb_t *policydb = &p->p;
 
 	head = policydb->ocontexts[OCON_NETIF];
-	for (c =  head; c != NULL; c = c->next) 
+	for (c = head; c != NULL; c = c->next)
 		count++;
 
 	*response = count;
@@ -239,17 +234,15 @@ extern int sepol_iface_count(
 	return STATUS_SUCCESS;
 }
 
-int sepol_iface_iterate(
-	sepol_handle_t* handle,
-	const sepol_policydb_t* p,
-	int (*fn)(
-		const sepol_iface_t* iface,
-		void* fn_arg),
-	void* arg) {
+int sepol_iface_iterate(sepol_handle_t * handle,
+			const sepol_policydb_t * p,
+			int (*fn) (const sepol_iface_t * iface,
+				   void *fn_arg), void *arg)
+{
 
 	const policydb_t *policydb = &p->p;
 	ocontext_t *c, *head;
-	sepol_iface_t* iface = NULL;
+	sepol_iface_t *iface = NULL;
 
 	head = policydb->ocontexts[OCON_NETIF];
 	for (c = head; c; c = c->next) {
@@ -257,7 +250,7 @@ int sepol_iface_iterate(
 
 		if (iface_to_record(handle, policydb, c, &iface) < 0)
 			goto err;
-		
+
 		/* Invoke handler */
 		status = fn(iface, arg);
 		if (status < 0)
@@ -273,9 +266,8 @@ int sepol_iface_iterate(
 
 	return STATUS_SUCCESS;
 
-	err:
+      err:
 	ERR(handle, "could not iterate over interfaces");
 	sepol_iface_free(iface);
 	return STATUS_ERR;
 }
-
